<?php
if (!defined('ABSPATH')) exit;

// 严格的权限检查
if (!is_admin() || !current_user_can('manage_options')) {
    wp_die(__('您没有足够的权限访问此页面'));
}

class JefRSSFriend_Reader_View {
    protected $db;
    protected $current_article;
    protected $articles;
    protected $font_size;
    protected $is_mobile_view = false;
    
    public function __construct($skip_redirect = false) {
        global $wpdb;
        $this->db = $wpdb;
        
        // 只在非移动端视图且未设置跳过重定向时执行重定向
        if (!$skip_redirect && !$this->is_mobile_view && wp_is_mobile() && !isset($_GET['no_redirect'])) {
            $mobile_url = add_query_arg(
                array(
                    'page' => 'jefrssfriend-reader-mobile',
                    'id' => isset($_GET['id']) ? $_GET['id'] : ''
                ),
                admin_url('admin.php')
            );
            wp_redirect($mobile_url);
            exit;
        }
        
        $this->init();
    }
    
    private function init() {
        // 获取当前文章ID
        $article_id = isset($_GET['id']) ? intval($_GET['id']) : 0;
        
        // 获取最新的文章列表
        $this->articles = $this->db->get_results("
            SELECT 
                a.*, 
                s.title as site_title,
                s.site_url,
                s.icon_url
            FROM {$this->db->prefix}jefrss_articles a
            LEFT JOIN {$this->db->prefix}jefrss_sites s ON a.site_id = s.id
            WHERE s.status = 'active'
            ORDER BY a.pub_date DESC
            LIMIT 50
        ");
        
        // 如果有指定文章ID，获取当前文章
        if ($article_id > 0) {
            $this->current_article = $this->db->get_row($this->db->prepare("
                SELECT 
                    a.*, 
                    s.title as site_title, 
                    s.site_url,
                    s.icon_url
                FROM {$this->db->prefix}jefrss_articles a
                LEFT JOIN {$this->db->prefix}jefrss_sites s ON a.site_id = s.id
                WHERE a.id = %d
            ", $article_id));
        }
    }
    
    // AJAX获取文章内容
    public function get_article_ajax() {
        check_ajax_referer('jefrssfriend-admin');
        
        $article_id = intval($_POST['id']);
        
        $article = $this->db->get_row($this->db->prepare("
            SELECT a.*, s.title as site_title, s.site_url, s.icon_url
            FROM {$this->db->prefix}jefrss_articles a
            LEFT JOIN {$this->db->prefix}jefrss_sites s ON a.site_id = s.id
            WHERE a.id = %d
        ", $article_id));
        
        if ($article) {
            $content = $this->get_article_template($article);
            wp_send_json_success(['content' => $content]);
        } else {
            wp_send_json_error('文章不存在');
        }
    }
    
    // 获取文章内容模板
    protected function get_article_template($article) {
        ob_start();
        ?>
        <div class="article-container">
            <h1>
                <a href="<?php echo esc_url($article->link); ?>" target="_blank">
                    <?php echo esc_html($article->title); ?>
                </a>
            </h1>
            <div class="jef-meta-info">
                <div class="site-info">
                    <img src="<?php echo esc_url($this->get_site_icon($article)); ?>" 
                         class="site-icon" 
                         alt="<?php echo esc_attr($article->site_title); ?>">
                    <span class="site-title"><?php echo esc_html($article->site_title); ?></span>
                </div>
                <time><?php echo human_time_diff(strtotime($article->pub_date)) . __('前', 'jefrssfriend'); ?></time>
                <button class="toggle-dark-mode" title="<?php _e('切换夜间模式', 'jefrssfriend'); ?>">
                    <svg viewBox="0 0 24 24" width="18" height="18">
                        <path d="M12 3a9 9 0 1 0 9 9c0-.46-.04-.92-.1-1.36a5.389 5.389 0 0 1-4.4 2.26 5.403 5.403 0 0 1-3.14-9.8c-.44-.06-.9-.1-1.36-.1z"/>
                    </svg>
                </button>
                <div class="jef-font-control desktop-only">
                    <button data-size="small" class="jef-size-btn">小</button>
                    <button data-size="medium" class="jef-size-btn">中</button>
                    <button data-size="large" class="jef-size-btn">大</button>
                </div>
            </div>
            <div class="jef-article-content">
                <?php echo wp_kses_post($article->content); ?>
            </div>
            <div class="article-footer">
                <a href="<?php echo esc_url($article->link); ?>" 
                   class="original-link" 
                   target="_blank">
                    <?php _e('访问原文', 'jefrssfriend'); ?> →
                </a>
            </div>
        </div>
        <?php
        return ob_get_clean();
    }
    
    // 获取站点图标
    protected function get_site_icon($article) {
        if (!empty($article->icon_url)) {
            return $article->icon_url;
        }
        return jefrssfriend_get_icon_service_url(parse_url($article->site_url, PHP_URL_HOST));
    }
    
    // 渲染页面
    public function render() {
        ?>
        <div class="wrap">
            <div class="reader-center">
                <!-- 左侧文章列表 -->
                <div class="articles-list">
                    <div class="list-header">
                        <h2><?php _e('最新文章', 'jefrssfriend'); ?></h2>
                    </div>
                    <div class="list-content">
                        <?php foreach ($this->articles as $article): ?>
                            <div class="article-item<?php echo ($this->current_article && $article->id == $this->current_article->id) ? ' active' : ''; ?>" 
                                 data-id="<?php echo esc_attr($article->id); ?>">
                                <img src="<?php echo esc_url($this->get_site_icon($article)); ?>" 
                                     class="site-icon" 
                                     alt="<?php echo esc_attr($article->site_title); ?>">
                                <h3><?php echo esc_html($article->title); ?></h3>
                                <div class="article-meta">
                                    <span class="site-title"><?php echo esc_html($article->site_title); ?></span>
                                    <time><?php echo human_time_diff(strtotime($article->pub_date)) . __('前', 'jefrssfriend'); ?></time>
                                </div>
                                <div class="article-desc">
                                    <?php echo wp_trim_words(wp_strip_all_tags($article->content), 55, '...'); ?>
                                </div>
                            </div>
                        <?php endforeach; ?>
                    </div>
                </div>

                <!-- 右侧阅读区域 -->
                <div class="reader-content">
                    <?php 
                    if ($this->current_article) {
                        echo $this->get_article_template($this->current_article);
                    } else {
                        echo '<div class="notice notice-info"><p>' . __('请从左侧选择要阅读的文章', 'jefrssfriend') . '</p></div>';
                    }
                    ?>
                </div>
            </div>
            
            <!-- 返回顶部按钮 - 放在reader-center外部 -->
            <button class="back-to-top" aria-label="返回顶部">
                <svg viewBox="0 0 24 24" width="24" height="24">
                    <path d="M12 4l-8 8h16z"></path>
                </svg>
            </button>
        </div>
        <?php
    }
} 