<?php
if (!defined('ABSPATH')) exit;

/**
 * 全局错误日志数组
 */
global $jefrssfriend_error_logs;
$jefrssfriend_error_logs = array();

/**
 * 刷新所有RSS源
 */
function jefrssfriend_refresh_all_feeds() {
    global $wpdb;
    $processed = 0;
    $failed = 0;
    
    try {
        $sites = $wpdb->get_results(
            "SELECT * FROM {$wpdb->prefix}jefrss_sites WHERE status = 'active'"
        );
        
        foreach (array_chunk($sites, 5) as $batch) {
            foreach ($batch as $site) {
                try {
                    jefrssfriend_refresh_single_feed($site);
                    $processed++;
                } catch (Exception $e) {
                    jefrssfriend_log("刷新失败: {$site->rss_url} - " . $e->getMessage());
                    $failed++;
                }
            }
        }
        
        // 只生成latest.json
        jefrssfriend_generate_latest_json();
        
        return array(
            'processed' => $processed,
            'failed' => $failed,
            'total' => count($sites)
        );
        
    } catch (Exception $e) {
        jefrssfriend_log("刷新全部失败: " . $e->getMessage());
        throw $e;
    }
}

/**
 * 刷新单个RSS源
 */
function jefrssfriend_refresh_single_feed($site) {
    try {
        // 验证站点对象
        if (!is_object($site) || empty($site->id) || empty($site->rss_url)) {
            throw new Exception('无效的站点数据');
        }
        
        // 获取设置的文章数量限制
        $options = get_option('jefrssfriend_options');
        $articles_per_feed = isset($options['articles_per_feed']) ? intval($options['articles_per_feed']) : 5;
        
        // 设置数据库编码
        global $wpdb;
        $wpdb->query("SET NAMES utf8mb4");
        $wpdb->query("SET CHARACTER SET utf8mb4");
        
        // 获取RSS内容
        $rss = fetch_feed($site->rss_url);
        if (is_wp_error($rss)) {
            throw new Exception($rss->get_error_message());
        }
        
        // 获取文章列表
        $items = $rss->get_items(0, $articles_per_feed);
        $processed = 0;
        
        foreach ($items as $item) {
            try {
                // 修改哈希生成逻辑，使用更多唯一标识
                $article_hash = md5(
                    $site->id .                    // 站点ID
                    $item->get_permalink() .       // 文章链接
                    $item->get_date('Y-m-d H:i:s') // 发布时间
                );
                
                // 增加调试日志
                error_log(sprintf(
                    '检查文章是否存在 - 站点: %d, 链接: %s, 哈希: %s',
                    $site->id,
                    $item->get_permalink(),
                    $article_hash
                ));
                
                // 修改查询条件，同时检查站点ID和文章哈希
                $exists = $wpdb->get_var($wpdb->prepare(
                    "SELECT COUNT(*) FROM {$wpdb->prefix}jefrss_articles 
                    WHERE site_id = %d AND article_hash = %s",
                    $site->id,
                    $article_hash
                ));
                
                error_log('文章是否存在: ' . ($exists ? '是' : '否'));
                
                if (!$exists) {
                    // 保存新文章
                    $result = $wpdb->insert(
                        $wpdb->prefix . 'jefrss_articles',
                        array(
                            'title' => $item->get_title(),
                            'content' => $item->get_content(),
                            'site_id' => $site->id,
                            'pub_date' => $item->get_date('Y-m-d H:i:s'),
                            'link' => $item->get_permalink(),
                            'article_hash' => $article_hash
                        ),
                        array('%s', '%s', '%d', '%s', '%s', '%s')
                    );
                    
                    if ($result) {
                        $processed++;
                        error_log('成功保存新文章: ' . $item->get_title());
                    }
                }
            } catch (Exception $e) {
                error_log('处理文章失败: ' . $e->getMessage());
                continue;
            }
        }
        
        // 更新站点状态
        $wpdb->update(
            $wpdb->prefix . 'jefrss_sites',
            array('last_fetch' => current_time('mysql')),
            array('id' => $site->id)
        );
        
        return array(
            'success' => true,
            'data' => array(
                'processed' => $processed,
                'total' => count($items)
            )
        );
        
    } catch (Exception $e) {
        error_log('RSS刷新错误: ' . $e->getMessage());
        return array(
            'success' => false,
            'message' => $e->getMessage()
        );
    }
}

/**
 * 清理指定站点的旧文章
 */
function jefrssfriend_cleanup_old_articles($site_id) {
    global $wpdb;
    
    // 获取设置中的保留文章数量
    $options = get_option('jefrssfriend_options');
    $keep_count = isset($options['articles_per_feed']) ? intval($options['articles_per_feed']) : 10;
    
    // 删除多余的旧文章
    $wpdb->query($wpdb->prepare(
        "DELETE FROM {$wpdb->prefix}jefrss_articles 
         WHERE site_id = %d 
         AND id NOT IN (
             SELECT id FROM (
                 SELECT id 
                 FROM {$wpdb->prefix}jefrss_articles 
                 WHERE site_id = %d 
                 ORDER BY pub_date DESC 
                 LIMIT %d
             ) temp
         )",
        $site_id, $site_id, $keep_count
    ));
}

function validate_html_structure($content) {
    // 基本的HTML结构验证
    $open_tags = substr_count($content, '<');
    $close_tags = substr_count($content, '>');
    
    if ($open_tags !== $close_tags) {
        error_log("HTML标签不匹配: 开始标签 $open_tags, 结束标签 $close_tags");
        return false;
    }
    
    return true;
}

/**
 * 生成最新文章JSON
 */
function jefrssfriend_generate_latest_json() {
    try {
        global $wpdb;
        
        // 获取最新文章
        $articles = $wpdb->get_results("
            SELECT a.*, s.title as site_title, s.icon_url
            FROM {$wpdb->prefix}jefrss_articles a
            JOIN {$wpdb->prefix}jefrss_sites s ON a.site_id = s.id
            WHERE s.status = 'active'
            ORDER BY a.pub_date DESC
            LIMIT 100
        ");
        
        // 准备JSON数据
        $json_data = array(
            'last_update' => current_time('timestamp'),
            'articles' => array_map(function($article) {
                return array(
                    'id' => md5($article->link),
                    'title' => $article->title,
                    'url' => $article->link,
                    'date' => $article->pub_date,
                    'site_name' => $article->site_title,
                    'site_icon' => $article->icon_url
                );
            }, $articles)
        );
        
        // 写入文件
        $json_file = JEFRSSFRIEND_PATH . 'data/latest.json';
        return file_put_contents($json_file, json_encode($json_data));
        
    } catch (Exception $e) {
        error_log('生成latest.json失败: ' . $e->getMessage());
        return false;
    }
}

/**
 * 获取所有文章
 */
function jefrssfriend_get_all_articles($with_full_content = false) {
    global $wpdb;
    $articles = array();
    
    // 获取所有站点
    $sites = $wpdb->get_results(
        "SELECT * FROM {$wpdb->prefix}jefrss_sites WHERE status = 'active'"
    );
    
    foreach ($sites as $site) {
        $dir_index = floor($site->id / 100);
        $json_path = JEFRSSFRIEND_PATH . "data/articles/{$dir_index}00-{$dir_index}99/site-{$site->id}.json";
        
        if (file_exists($json_path)) {
            $data = json_decode(file_get_contents($json_path), true);
            foreach ($data['articles'] as $article) {
                $article['site_name'] = $site->title;
                $article['site_icon'] = jefrssfriend_get_site_icon($site);
                $article['site_id'] = $site->id;
                $article['group_id'] = $site->group_id;
                
                if (!$with_full_content) {
                    unset($article['content']);
                    unset($article['images']);
                }
                
                $articles[] = $article;
            }
        }
    }
    
    return $articles;
}

/**
 * 记录日志
 */
function jefrssfriend_log($message) {
    if (WP_DEBUG && defined('JEFRSSFRIEND_DEBUG') && JEFRSSFRIEND_DEBUG) {
        error_log('[JefRSS] ' . $message);
    }
}

/**
 * 获取图标服务URL
 * @param string $domain 域名
 * @return string 图标URL
 */
function jefrssfriend_get_icon_service_url($domain) {
    // 获取设置中的服务商选项
    $options = get_option('jefrssfriend_options', array());
    $service = isset($options['icon_service']) ? $options['icon_service'] : 'unavatar';
    
    // 确保域名干净
    $domain = trim($domain);
    
    // 根据不同服务商返回对应的URL
    switch($service) {
        case 'favicon':
            return 'https://favicon.im/' . $domain;
        case 'cravatar':
            return 'https://cravatar.com/favicon/api/index.php?url=' . $domain;
        case 'afmax':
            return 'https://api.afmax.cn/so/ico/index.php?r=' . $domain;
        case 'iconhorse':
            return 'https://icon.horse/icon/' . $domain;
        case 'unavatar':
        default:
            return 'https://unavatar.io/' . $domain;
    }
}

/**
 * 修改现有的获取站点图标函数使用新的服务
 */
function jefrssfriend_get_site_icon($site) {
    // 优先使用自定义图标
    if (!empty($site->icon_url)) {
        return $site->icon_url;
    }
    
    // 如果有站点URL,使用图标服务
    if (!empty($site->site_url)) {
        $domain = parse_url($site->site_url, PHP_URL_HOST);
        if (!empty($domain)) {
            return jefrssfriend_get_icon_service_url($domain);
        }
    }
    
    // 默认图标
    return JEFRSSFRIEND_URL . 'assets/images/default-icon.png';
}

/**
 * 获取默认站点图标
 */
function jefrssfriend_get_default_icon() {
    return JEFRSSFRIEND_URL . 'assets/images/default-icon.png';
}

/**
 * 提取文章中的第一张图片
 */
function jefrssfriend_get_first_image($content) {
    if (preg_match('/<img.+?src=[\'"]([^\'"]+)[\'"].*?>/i', $content, $matches)) {
        $image_url = $matches[1];
        
        // 检查图片URL是否有效
        $response = wp_remote_head($image_url, array(
            'timeout' => 5,
            'sslverify' => false
        ));
        
        // 只有当图片可访问时才返回URL
        if (!is_wp_error($response) && 
            wp_remote_retrieve_response_code($response) === 200 &&
            strpos(wp_remote_retrieve_header($response, 'content-type'), 'image/') === 0) {
            return $image_url;
        }
    }
    
    return '';
}

/**
 * 获取文章摘要
 * @param string $content 文章内容
 * @param int $length 摘要长度
 * @return string 处理后的摘要
 */
function jefrssfriend_get_excerpt($content, $length) {
    // 生成缓存键
    $cache_key = 'jefrss_excerpt_' . md5($content) . '_' . $length;
    
    // 尝试获取缓存
    $excerpt = get_transient($cache_key);
    if ($excerpt !== false) {
        return $excerpt;
    }
    
    // 先去除HTML标签
    $text = strip_tags($content);
    
    // 去除多余的空白字符
    $text = preg_replace('/\s+/', ' ', $text);
    
    // 截取指定长度
    if (function_exists('mb_substr')) {
        $excerpt = mb_substr($text, 0, $length);
    } else {
        $excerpt = substr($text, 0, $length);
    }
    
    // 如果内容被截断，添加省略号
    if (strlen($text) > $length) {
        $excerpt .= '...';
    }
    
    // 缓存结果（1小时）
    set_transient($cache_key, $excerpt, HOUR_IN_SECONDS);
    
    return $excerpt;
}

/**
 * 优化文章内容
 */
function jefrssfriend_optimize_content($article_id) {
    global $wpdb;
    
    try {
        // 获取文章
        $article = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM {$wpdb->prefix}jefrss_articles WHERE id = %d",
            $article_id
        ));
        
        if (!$article) {
            throw new Exception('文章不存在');
        }
        
        // 处理内容
        $updater = new JefRSSFriend_Lite_Updater();
        $result = $updater->process_article_content($article->content);
        
        if (!$result) {
            throw new Exception('内容处理失败');
        }
        
        // 更新文章
        $updated = $wpdb->update(
            $wpdb->prefix . 'jefrss_articles',
            array(
                'content' => $result['content'],
                'title' => $result['title'],
                'processed' => 1
            ),
            array('id' => $article_id)
        );
        
        if ($updated === false) {
            throw new Exception('更新数据库失败');
        }
        
        return $result;
        
    } catch (Exception $e) {
        error_log('Content optimization error: ' . $e->getMessage());
        return false;
    }
}

/**
 * 获取优化后的文章内容
 */
function jefrssfriend_get_optimized_content($article_id) {
    global $wpdb;
    
    return $wpdb->get_row($wpdb->prepare(
        "SELECT * FROM {$wpdb->prefix}jefrss_articles WHERE id = %d",
        $article_id
    ));
}

/**
 * 检查并修复数据库设置
 */
function jefrssfriend_check_db_settings() {
    global $wpdb;
    
    // 1. 检查数据库连接字符集
    $charset_collate = $wpdb->get_charset_collate();
    error_log('数据库字符集: ' . $charset_collate);
    
    // 2. 检查表结构
    $table_name = $wpdb->prefix . 'jefrss_articles';
    $table_info = $wpdb->get_results("SHOW CREATE TABLE $table_name");
    error_log('表结构: ' . print_r($table_info, true));
    
    // 3. 修改表字段为LONGTEXT
    $wpdb->query("ALTER TABLE $table_name MODIFY COLUMN content LONGTEXT CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci");
    
    // 4. 设置连接字符集
    $wpdb->query("SET NAMES utf8mb4");
    $wpdb->query("SET CHARACTER SET utf8mb4");
    
    return true;
}

/**
 * 保存文章到数据库
 */
function jefrssfriend_save_article($article_data) {
    global $wpdb;
    
    try {
        // 使用原生SQL插入
        $sql = $wpdb->prepare(
            "INSERT INTO {$wpdb->prefix}jefrss_articles 
            (title, content, site_id, pub_date, link, article_hash, processed)
            VALUES (%s, %s, %d, %s, %s, %s, %d)",
            $article_data['title'],
            $article_data['content'],  // 直接使用原始内容
            $article_data['site_id'],
            $article_data['pub_date'],
            $article_data['link'],
            $article_data['article_hash'],
            $article_data['processed']
        );
        
        $result = $wpdb->query($sql);
        
        if ($result === false) {
            throw new Exception($wpdb->last_error);
        }
        
        // 验证保存的内容
        $saved = $wpdb->get_row($wpdb->prepare(
            "SELECT content FROM {$wpdb->prefix}jefrss_articles WHERE id = %d",
            $wpdb->insert_id
        ));
        
        // 记录保存前后的内容对比
        error_log('保存前HTML标签数: ' . substr_count($article_data['content'], '<'));
        error_log('保存后HTML标签数: ' . substr_count($saved->content, '<'));
        
        return $wpdb->insert_id;
    } catch (Exception $e) {
        error_log('保存文章失败: ' . $e->getMessage());
        return false;
    }
}

function jefrssfriend_get_article_content($article_id) {
    global $wpdb;
    
    $article = $wpdb->get_row($wpdb->prepare(
        "SELECT * FROM {$wpdb->prefix}jefrss_articles WHERE id = %d",
        $article_id
    ));
    
    // 直接返回原始内容，不做任何解码
    return $article;
}

function jefrssfriend_debug_content($article_id) {
    global $wpdb;
    
    $article = $wpdb->get_row($wpdb->prepare(
        "SELECT * FROM {$wpdb->prefix}jefrss_articles WHERE id = %d",
        $article_id
    ));
    
    if ($article) {
        error_log('文章ID: ' . $article_id);
        error_log('原始内容: ' . $article->content);
        error_log('处理状态: ' . $article->processed);
    }
}

/**
 * 检查数据库字符集设置
 */
function jefrssfriend_check_charset() {
    global $wpdb;
    
    // 获取WordPress配置的字符集
    $wp_charset = defined('DB_CHARSET') ? DB_CHARSET : '';
    $wp_collate = defined('DB_COLLATE') ? DB_COLLATE : '';
    
    // 获取数据库实际字符集
    $db_charset = $wpdb->get_var("SELECT @@character_set_database");
    $db_collate = $wpdb->get_var("SELECT @@collation_database");
    
    // 获取表字符集
    $table_name = $wpdb->prefix . 'jefrss_articles';
    $table_info = $wpdb->get_row("SHOW CREATE TABLE $table_name", ARRAY_N);
    
    error_log("WordPress配置字符集: $wp_charset");
    error_log("WordPress配置排序规则: $wp_collate");
    error_log("数据库字符集: $db_charset");
    error_log("数据库排序规则: $db_collate");
    error_log("表创建语句: " . $table_info[1]);
    
    return array(
        'wp_charset' => $wp_charset,
        'wp_collate' => $wp_collate,
        'db_charset' => $db_charset,
        'db_collate' => $db_collate,
        'table_info' => $table_info[1]
    );
}

/**
 * 记录错误日志
 */
function jefrssfriend_log_error($message) {
    global $jefrssfriend_error_logs;
    $jefrssfriend_error_logs[] = $message;
    error_log($message);
}

/**
 * 测试不同方式保存内容
 */
function jefrssfriend_test_save_content($content) {
    global $wpdb, $jefrssfriend_error_logs;
    
    // 1. 创建临时测试表
    $wpdb->query("
        CREATE TABLE IF NOT EXISTS {$wpdb->prefix}jefrss_test (
            id INT AUTO_INCREMENT PRIMARY KEY,
            save_method VARCHAR(50),
            content_type VARCHAR(50),
            original_length INT,
            saved_length INT,
            content LONGTEXT,
            created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP
        ) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci
    ");
    
    // 2. 测试不同的保存方法
    $test_methods = array(
        // 方法1: 直接保存原始内容
        array(
            'method' => 'raw',
            'content' => $content
        ),
        
        // 方法2: JSON编码后保存
        array(
            'method' => 'json',
            'content' => json_encode($content)
        ),
        
        // 方法3: Base64编码后保存
        array(
            'method' => 'base64',
            'content' => base64_encode($content)
        ),
        
        // 方法4: 序列化后保存
        array(
            'method' => 'serialize',
            'content' => serialize($content)
        ),
        
        // 方法5: wp_kses过滤后保存
        array(
            'method' => 'wp_kses',
            'content' => wp_kses($content, wp_kses_allowed_html('post'))
        )
    );
    
    $results = array();
    
    foreach ($test_methods as $test) {
        // 保存内容
        $wpdb->insert(
            $wpdb->prefix . 'jefrss_test',
            array(
                'save_method' => $test['method'],
                'content_type' => gettype($test['content']),
                'original_length' => strlen($content),
                'saved_length' => strlen($test['content']),
                'content' => $test['content']
            ),
            array('%s', '%s', '%d', '%d', '%s')
        );
        
        // 立即读取保存的内容
        $saved = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM {$wpdb->prefix}jefrss_test WHERE id = %d",
            $wpdb->insert_id
        ));
        
        // 解码内容进行比较
        $decoded_content = $saved->content;
        if ($test['method'] === 'json') {
            $decoded_content = json_decode($saved->content);
        } elseif ($test['method'] === 'base64') {
            $decoded_content = base64_decode($saved->content);
        } elseif ($test['method'] === 'serialize') {
            $decoded_content = unserialize($saved->content);
        }
        
        // 记录结果
        $results[$test['method']] = array(
            'original_length' => strlen($content),
            'saved_length' => strlen($saved->content),
            'decoded_length' => strlen($decoded_content),
            'content_matches' => ($decoded_content === $content),
            'first_50_chars' => substr($decoded_content, 0, 50)
        );
        
        jefrssfriend_log_error("测试方法 {$test['method']}:");
        jefrssfriend_log_error("- 原始长度: " . strlen($content));
        jefrssfriend_log_error("- 保存长度: " . strlen($saved->content));
        jefrssfriend_log_error("- 解码长度: " . strlen($decoded_content));
        jefrssfriend_log_error("- 内容匹配: " . ($decoded_content === $content ? '是' : '否'));
        jefrssfriend_log_error("- 前50个字符: " . substr($decoded_content, 0, 50));
    }
    
    return $results;
}

function jefrssfriend_display_content($content) {
    // 添加调试日志
    error_log('显示内容HTML标签数: ' . substr_count($content, '<'));
    
    // 不进行任何过滤，直接返回原始内容
    return $content;
}

function jefrssfriend_should_filter_content() {
    // 从设置中获取是否需要过滤内容
    $options = get_option('jefrssfriend_options', array());
    return isset($options['filter_content']) ? $options['filter_content'] : false;
}

function jefrssfriend_validate_content($content) {
    // 检查是否包含HTML标签
    if (strip_tags($content) === $content) {
        error_log('警告：内容不包含HTML标签');
        return false;
    }
    
    // 检查HTML结构
    $dom = new DOMDocument();
    libxml_use_internal_errors(true);
    $result = $dom->loadHTML($content, LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
    libxml_clear_errors();
    
    if (!$result) {
        error_log('警告：HTML结构无效');
        return false;
    }
    
    return true;
}

function validate_saved_article($id) {
    global $wpdb;
    
    $article = $wpdb->get_row($wpdb->prepare(
        "SELECT content FROM {$wpdb->prefix}jefrss_articles WHERE id = %d",
        $id
    ));
    
    if (!$article) {
        error_log('文章不存在');
        return false;
    }
    
    // 检查HTML标签
    if (strip_tags($article->content) === $article->content) {
        error_log('警告：保存的内容丢失了HTML标签');
        return false;
    }
    
    return true;
}

function jefrssfriend_set_db_encoding() {
    global $wpdb;
    $wpdb->query("SET NAMES utf8mb4");
    $wpdb->query("SET CHARACTER SET utf8mb4");
}

function jefrssfriend_check_file_permissions() {
    $data_dir = JEFRSSFRIEND_PATH . 'data';
    
    if (!is_writable($data_dir)) {
        error_log('数据目录不可写: ' . $data_dir);
        return false;
    }
    
    $json_file = $data_dir . '/latest.json';
    if (file_exists($json_file) && !is_writable($json_file)) {
        error_log('JSON文件不可写: ' . $json_file);
        return false;
    }
    
    return true;
}

/**
 * 清理重复文章
 */
function jefrssfriend_cleanup_duplicate_articles() {
    global $wpdb;
    
    // 找出并删除重复文章，保留最新的一条
    $wpdb->query("
        DELETE a1 FROM {$wpdb->prefix}jefrss_articles a1
        INNER JOIN {$wpdb->prefix}jefrss_articles a2
        WHERE a1.id < a2.id  /* 保留最新的记录 */
        AND a1.site_id = a2.site_id
        AND a1.article_hash = a2.article_hash
    ");
    
    return $wpdb->rows_affected;
}